import pytest
from kiota_abstractions.serialization import SerializationWriterFactory

from kiota_serialization_text.text_serialization_writer import TextSerializationWriter
from kiota_serialization_text.text_serialization_writer_factory import (
    TextSerializationWriterFactory,
)


def test_get_serialization_writer():
    factory = TextSerializationWriterFactory()
    writer = factory.get_serialization_writer('text/plain')
    assert isinstance(writer, TextSerializationWriter)


def test_get_serialization_writer_no_content_type():
    with pytest.raises(TypeError) as e_info:
        factory = TextSerializationWriterFactory()
        writer = factory.get_serialization_writer("")


def test_get_serialization_writer_unsupported_content_type():
    with pytest.raises(Exception) as e_info:
        factory = TextSerializationWriterFactory()
        writer = factory.get_serialization_writer('application/xml')


def test_get_valid_content_type():
    factory = TextSerializationWriterFactory()
    content_type = factory.get_valid_content_type()
    assert content_type == 'text/plain'



import pytest

from kiota_serialization_text.text_parse_node import TextParseNode
from kiota_serialization_text.text_parse_node_factory import TextParseNodeFactory


@pytest.fixture
def sample_text_string():
    return 'Microsoft Graph'


def test_get_root_parse_node(sample_text_string):
    factory = TextParseNodeFactory()
    sample_text_string_bytes = sample_text_string.encode('utf-8')
    root = factory.get_root_parse_node('text/plain', sample_text_string_bytes)
    assert isinstance(root, TextParseNode)


def test_get_root_parse_node_no_content_type(sample_text_string):
    with pytest.raises(Exception) as e_info:
        factory = TextParseNodeFactory()
        sample_text_string_bytes = sample_text_string.encode('utf-8')
        root = factory.get_root_parse_node('', sample_text_string_bytes)


def test_get_root_parse_node_unsupported_content_type(sample_text_string):
    with pytest.raises(Exception) as e_info:
        factory = TextParseNodeFactory()
        sample_text_string_bytes = sample_text_string.encode('utf-8')
        root = factory.get_root_parse_node('application/xml', sample_text_string_bytes)


def test_get_root_parse_node_empty_text():
    with pytest.raises(TypeError) as e_info:
        factory = TextParseNodeFactory()
        sample_string_bytes = ''.encode('utf-8')
        root = factory.get_root_parse_node('text/plain', sample_string_bytes)


def test_get_valid_content_type():
    factory = TextParseNodeFactory()
    content_type = factory.get_valid_content_type()
    assert content_type == 'text/plain'



from uuid import UUID

import pytest
from dateutil import parser
from kiota_abstractions.serialization import Parsable, SerializationWriter

from kiota_serialization_text.text_serialization_writer import TextSerializationWriter

from ..helpers import OfficeLocation, User


@pytest.fixture
def user_1():
    user = User()
    user.age = 31
    user.is_active = True
    user.display_name = "Jane Doe"
    return user


@pytest.fixture
def user_2():
    user = User()
    user.age = 32
    user.is_active = False
    user.display_name = "John Doe"
    return user


def test_write_str_value():
    text_serialization_writer = TextSerializationWriter()
    text_serialization_writer.write_str_value("", "Adele Vance")
    content = text_serialization_writer.get_serialized_content()
    content_string = content.decode('utf-8')
    assert content_string == 'Adele Vance'


def test_write_str_value_with_key():
    with pytest.raises(Exception) as e_info:
        text_serialization_writer = TextSerializationWriter()
        text_serialization_writer.write_str_value("displayName", "Adele Vance")


def test_write_bool_value():
    text_serialization_writer = TextSerializationWriter()
    text_serialization_writer.write_bool_value(None, True)
    content = text_serialization_writer.get_serialized_content()
    content_string = content.decode('utf-8')
    assert content_string == 'true'


def test_write_bool_value_with_key():
    with pytest.raises(Exception) as e_info:
        text_serialization_writer = TextSerializationWriter()
        text_serialization_writer.write_bool_value("isActive", True)


def test_write_int_value():
    text_serialization_writer = TextSerializationWriter()
    text_serialization_writer.write_int_value("", 28192199291929192)
    content = text_serialization_writer.get_serialized_content()
    content_string = content.decode('utf-8')
    assert content_string == '28192199291929192'


def test_write_int_value_with_key():
    with pytest.raises(Exception) as e_info:
        text_serialization_writer = TextSerializationWriter()
        text_serialization_writer.write_int_value("timestamp", 28192199291929192)


def test_write_float_value():
    text_serialization_writer = TextSerializationWriter()
    text_serialization_writer.write_float_value("", 3.2)
    content = text_serialization_writer.get_serialized_content()
    content_string = content.decode('utf-8')
    assert content_string == '3.2'


def test_write_float_value_with_key():
    with pytest.raises(Exception) as e_info:
        text_serialization_writer = TextSerializationWriter()
        text_serialization_writer.write_float_value("gpa", 3.2)


def test_write_uuid_value():
    text_serialization_writer = TextSerializationWriter()
    text_serialization_writer.write_uuid_value("", UUID("8f841f30-e6e3-439a-a812-ebd369559c36"))
    content = text_serialization_writer.get_serialized_content()
    content_string = content.decode('utf-8')
    assert content_string == "8f841f30-e6e3-439a-a812-ebd369559c36"


def test_write_uuid_value_with_key():
    with pytest.raises(Exception) as e_info:
        text_serialization_writer = TextSerializationWriter()
        text_serialization_writer.write_uuid_value(
            "id", UUID("8f841f30-e6e3-439a-a812-ebd369559c36")
        )


def test_write_datetime_value():
    text_serialization_writer = TextSerializationWriter()
    text_serialization_writer.write_datetime_value("", parser.parse('2022-01-27T12:59:45.596117'))
    content = text_serialization_writer.get_serialized_content()
    content_string = content.decode('utf-8')
    assert content_string == "2022-01-27T12:59:45.596117"


def test_write_datetime_value_with_key():
    with pytest.raises(Exception) as e_info:
        text_serialization_writer = TextSerializationWriter()
        text_serialization_writer.write_datetime_value(
            "updatedAt", parser.parse('2022-01-27T12:59:45.596117')
        )


def test_write_timedelta_value():
    text_serialization_writer = TextSerializationWriter()
    text_serialization_writer.write_timedelta_value(
        "",
        parser.parse('2022-01-27T12:59:45.596117') - parser.parse('2022-01-27T10:59:45.596117')
    )
    content = text_serialization_writer.get_serialized_content()
    content_string = content.decode('utf-8')
    assert content_string == "2:00:00"


def test_write_timedelta_value_with_key():
    with pytest.raises(Exception) as e_info:
        text_serialization_writer = TextSerializationWriter()
        text_serialization_writer.write_timedelta_value(
            "diff",
            parser.parse('2022-01-27T12:59:45.596117') - parser.parse('2022-01-27T10:59:45.596117')
        )


def test_write_date_value():
    text_serialization_writer = TextSerializationWriter()
    text_serialization_writer.write_date_value("", parser.parse("2000-09-04").date())
    content = text_serialization_writer.get_serialized_content()
    content_string = content.decode('utf-8')
    assert content_string == "2000-09-04"


def test_write_date_value_with_key():
    with pytest.raises(Exception) as e_info:
        text_serialization_writer = TextSerializationWriter()
        text_serialization_writer.write_date_value("birthday", parser.parse("2000-09-04").date())


def test_write_time_value():
    text_serialization_writer = TextSerializationWriter()
    text_serialization_writer.write_time_value(
        "",
        parser.parse('2022-01-27T12:59:45.596117').time()
    )
    content = text_serialization_writer.get_serialized_content()
    content_string = content.decode('utf-8')
    assert content_string == "12:59:45.596117"


def test_write_time_value_with_key():
    with pytest.raises(Exception) as e_info:
        text_serialization_writer = TextSerializationWriter()
        text_serialization_writer.write_time_value(
            "time",
            parser.parse('2022-01-27T12:59:45.596117').time()
        )


def test_write_collection_of_primitive_values():
    with pytest.raises(Exception) as e_info:
        text_serialization_writer = TextSerializationWriter()
        text_serialization_writer.write_collection_of_primitive_values(
            "businessPhones", ["+1 412 555 0109", 1]
        )


def test_write_collection_of_object_values(user_1, user_2):
    with pytest.raises(Exception) as e_info:
        text_serialization_writer = TextSerializationWriter()
        text_serialization_writer.write_collection_of_object_values("users", [user_1, user_2])


def test_write_collection_of_enum_values():
    with pytest.raises(Exception) as e_info:
        text_serialization_writer = TextSerializationWriter()
        text_serialization_writer.write_collection_of_enum_values(
            "officeLocation", [OfficeLocation.Dunhill, OfficeLocation.Oval]
        )


def test_write_object_value(user_1):
    with pytest.raises(Exception) as e_info:
        text_serialization_writer = TextSerializationWriter()
        text_serialization_writer.write_object_value("user1", user_1)


def test_write_enum_value():
    text_serialization_writer = TextSerializationWriter()
    text_serialization_writer.write_enum_value("", OfficeLocation.Dunhill)
    content = text_serialization_writer.get_serialized_content()
    content_string = content.decode('utf-8')
    assert content_string == "dunhill"


def test_write_enum_value_with_key():
    with pytest.raises(Exception) as e_info:
        text_serialization_writer = TextSerializationWriter()
        text_serialization_writer.write_enum_value("officeLocation", OfficeLocation.Dunhill)


def test_write_null_value():
    text_serialization_writer = TextSerializationWriter()
    text_serialization_writer.write_null_value("")
    content = text_serialization_writer.get_serialized_content()
    content_string = content.decode('utf-8')
    assert content_string == 'null'


def test_write_null_value_with_key():
    with pytest.raises(Exception) as e_info:
        text_serialization_writer = TextSerializationWriter()
        text_serialization_writer.write_null_value("mobilePhone")


def test_write_additional_data_value():
    with pytest.raises(Exception) as e_info:
        text_serialization_writer = TextSerializationWriter()
        text_serialization_writer.write_additional_data_value(
            {
                "@odata.context": "https://graph.microsoft.com/v1.0/$metadata#users/$entity",
                "businessPhones": ["+1 205 555 0108"],
            }
        )



import json
from datetime import date, datetime, time, timedelta
from uuid import UUID

import pytest

from kiota_serialization_text.text_parse_node import TextParseNode

from ..helpers import OfficeLocation, User


@pytest.fixture
def sample_structured_data():
    user_json = json.dumps(
        {
            "@odata.context": "https://graph.microsoft.com/v1.0/$metadata#users/$entity",
            "businessPhones": ["+1 425 555 0109"],
            "displayName": "Adele Vance",
            "mobilePhone": None,
            "officeLocation": "dunhill",
            "updatedAt": "2017 -07-29T03:07:25Z",
            "birthday": "2000-09-04",
            "isActive": True,
            "age": 21,
            "gpa": 3.7,
            "id": "76cabd60-f9aa-4d23-8958-64f5539b826a"
        }
    )
    return str(user_json)


def test_get_str_value():
    parse_node = TextParseNode("Diego Siciliani")
    result = parse_node.get_str_value()
    assert result == "Diego Siciliani"


def test_get_int_value():
    parse_node = TextParseNode("1454")
    result = parse_node.get_int_value()
    assert result == 1454


def test_get_bool_value():
    parse_node = TextParseNode(False)
    result = parse_node.get_bool_value()
    assert result is False


def test_get_float_value():
    parse_node = TextParseNode(44.6)
    result = parse_node.get_float_value()
    assert result == 44.6


def test_get_uuid_value():
    parse_node = TextParseNode("f58411c7-ae78-4d3c-bb0d-3f24d948de41")
    result = parse_node.get_uuid_value()
    assert result == UUID("f58411c7-ae78-4d3c-bb0d-3f24d948de41")


def test_get_datetime_value():
    parse_node = TextParseNode('2022-01-27T12:59:45.596117')
    result = parse_node.get_datetime_value()
    assert isinstance(result, datetime)


def test_get_date_value():
    parse_node = TextParseNode('2015-04-20T11:50:51Z')
    result = parse_node.get_date_value()
    assert isinstance(result, date)
    assert str(result) == '2015-04-20'


def test_get_time_value():
    parse_node = TextParseNode('2022-01-27T12:59:45.596117')
    result = parse_node.get_time_value()
    assert isinstance(result, time)
    assert str(result) == '12:59:45.596117'


def test_get_timedelta_value():
    parse_node = TextParseNode('2022-01-27T12:59:45.596117')
    result = parse_node.get_timedelta_value()
    assert isinstance(result, timedelta)
    assert str(result) == '12:59:45'


def test_get_collection_of_primitive_values():
    with pytest.raises(Exception) as e_info:
        parse_node = TextParseNode([12.1, 12.2, 12.3, 12.4, 12.5])
        result = parse_node.get_collection_of_primitive_values(float)


def test_get_bytes_value():
    parse_node = TextParseNode('U2Ftd2VsIGlzIHRoZSBiZXN0')
    result = parse_node.get_bytes_value()
    assert isinstance(result, bytes)


def test_get_collection_of_enum_values():
    with pytest.raises(Exception) as e_info:
        parse_node = TextParseNode("dunhill,oval")
        result = parse_node.get_collection_of_enum_values(OfficeLocation)


def test_get_enum_value():
    parse_node = TextParseNode("dunhill")
    result = parse_node.get_enum_value(OfficeLocation)
    assert isinstance(result, OfficeLocation)
    assert result == OfficeLocation.Dunhill


def test_get_object_value(sample_structured_data):
    with pytest.raises(Exception) as e_info:
        parse_node = TextParseNode(sample_structured_data)
        result = parse_node.get_object_value(User)


def test_get_collection_of_object_values(sample_structured_data):
    with pytest.raises(Exception) as e_info:
        parse_node = TextParseNode(sample_structured_data)
        result = parse_node.get_collection_of_object_values(User)
